@@ -1,3 +1,13 @@
+def _bm_set_key(db, n, x, iters, threads, v):
+    for z in range(x * int(iters / threads), (x + 1) * int(iters / threads)):
+        db.set(key=f'.benchmark/{n}/key{z}', value=v)
+
+
+def _bm_get_key(db, n, x, iters, threads):
+    for z in range(x * int(iters / threads), (x + 1) * int(iters / threads)):
+        db.get(key=f'.benchmark/{n}/key{z}')
+
+
 def cli():
 
     import yedb
@@ -120,6 +130,8 @@ def cli():
 
     options = {}
 
+    ap = icli.ArgumentParser()
+
     try:
         db_dir = sys.argv[1]
         if db_dir.startswith('-'):
@@ -134,7 +146,6 @@ def cli():
                 'copy',
                 'rename',
                 'dump',
-                'load',
                 'ls',
                 'info',
                 'benchmark',
@@ -146,8 +157,10 @@ def cli():
         ]:
             sys.argv[1] = '-h'
             raise ValueError
-        if db_dir.startswith('http://') or db_dir.startswith('https://'):
-            options['http_timeout'] = 3600
+        if db_dir.startswith('http://') or db_dir.startswith(
+                'https://') or Path(db_dir).is_socket() or db_dir.endswith(
+                    '.sock') or db_dir.endswith('.socket'):
+            options['timeout'] = 60
             remote = True
             db_path = db_dir
             db_ps = db_dir.split('//', 1)[-1]
@@ -306,55 +319,85 @@ def cli():
             elif cmd == 'cat':
                 dispatcher(cmd='get', KEY=kwargs.get('KEY'), raw=True)
             elif cmd == 'dump':
-                import msgpack
-                key = kwargs.get('KEY')
-                if kwargs.get('FILE') == '-':
-                    f = None
-                    if sys.stdout.isatty():
-                        raise RuntimeError('stdout is a tty')
-                else:
-                    f = open(kwargs.get('FILE'), 'wb')
-                fd = f if f else sys.stdout.buffer
-                c = 0
-                try:
-                    for v in db.dump_keys(key=key):
-                        data = msgpack.dumps(v)
-                        fd.write(len(data).to_bytes(4, 'little') + data)
-                        c += 1
-                finally:
-                    if f:
-                        f.close()
-                print(f'{c} subkeys of {key} dumped')
-            elif cmd == 'load':
-                import msgpack
-                if kwargs.get('FILE') == '-':
-                    f = None
-                    if sys.stdin.isatty():
-                        raise RuntimeError('stdin is a tty')
-                else:
-                    f = open(kwargs.get('FILE'), 'rb')
-                fd = f if f else sys.stdin.buffer
-                buf = []
-                c = 0
-                try:
-                    while True:
-                        l = fd.read(4)
-                        if not l:
-                            break
-                        data = msgpack.loads(fd.read(int.from_bytes(
-                            l, 'little')),
-                                             raw=False)
-                        buf.append(data)
-                        c += 1
-                        if sys.getsizeof(buf) > 32768:
+                func = kwargs.get('_func')
+                if func is None:
+                    ap.print_help()
+                elif func == 'save':
+                    import msgpack
+                    key = kwargs.get('KEY')
+                    if kwargs.get('FILE') == '-':
+                        f = None
+                        if sys.stdout.isatty():
+                            raise RuntimeError('stdout is a tty')
+                    else:
+                        f = open(kwargs.get('FILE'), 'wb')
+                    fd = f if f else sys.stdout.buffer
+                    c = 0
+                    try:
+                        for v in db.dump_keys(key=key):
+                            data = msgpack.dumps(v)
+                            fd.write(len(data).to_bytes(4, 'little') + data)
+                            c += 1
+                    finally:
+                        if f:
+                            f.close()
+                    print(f'{c} subkey(s) of {key} dumped')
+                elif func == 'load':
+                    import msgpack
+                    if kwargs.get('FILE') == '-':
+                        f = None
+                        if sys.stdin.isatty():
+                            raise RuntimeError('stdin is a tty')
+                    else:
+                        f = open(kwargs.get('FILE'), 'rb')
+                    fd = f if f else sys.stdin.buffer
+                    buf = []
+                    c = 0
+                    try:
+                        while True:
+                            l = fd.read(4)
+                            if not l:
+                                break
+                            data = msgpack.loads(fd.read(int.from_bytes(
+                                l, 'little')),
+                                                 raw=False)
+                            buf.append(data)
+                            c += 1
+                            if sys.getsizeof(buf) > 32768:
+                                db.load_keys(data=buf)
+                                buf.clear()
+                        if buf:
                             db.load_keys(data=buf)
-                            buf.clear()
-                    if buf:
-                        db.load_keys(data=buf)
-                finally:
-                    if f:
-                        f.close()
-                print(f'{c} keys loaded')
+                    finally:
+                        if f:
+                            f.close()
+                    print(f'{c} key(s) loaded')
+                elif func == 'view':
+                    import msgpack
+                    import json
+                    full = kwargs.get('full')
+                    if kwargs.get('FILE') == '-':
+                        f = None
+                        if sys.stdin.isatty():
+                            raise RuntimeError('stdin is a tty')
+                    else:
+                        f = open(kwargs.get('FILE'), 'rb')
+                    fd = f if f else sys.stdin.buffer
+                    try:
+                        while True:
+                            l = fd.read(4)
+                            if not l:
+                                break
+                            data = msgpack.loads(fd.read(int.from_bytes(
+                                l, 'little')),
+                                                 raw=False)
+                            if full:
+                                print(json.dumps(data))
+                            else:
+                                print(data[0])
+                    finally:
+                        if f:
+                            f.close()
             elif cmd == 'copy':
                 db.copy(key=kwargs.get('KEY'),
                         dst_key=kwargs.get('DST_KEY'),
@@ -596,15 +639,28 @@ def cli():
                                  sum(f.stat().st_size for f in db_files))))
                 pretty_print_table(sorted(data, key=lambda k: k['name']))
             elif cmd == 'benchmark':
+
                 db.delete(key='.benchmark', recursive=True)
                 iters = 1000
+                threads = kwargs.get('threads', 4)
+                print(f'Benchmarking. Threads: {threads}')
+                print()
+                from concurrent.futures import ThreadPoolExecutor
+                p = ThreadPoolExecutor(max_workers=threads)
+                tasks = []
                 test_arr = [777.777] * 100
                 test_dict = {f'v{n}': n * 777.777 for n in range(100)}
                 for n, v in [('numeric', 777.777), ('string', 'x' * 1000),
                              ('array', test_arr), ('dict', test_dict)]:
                     start = time.perf_counter()
-                    for z in range(iters):
-                        db.set(key=f'.benchmark/{n}/key{z}', value=v)
+
+                    for i in range(threads):
+                        tasks.append(
+                            p.submit(_bm_set_key, db, n, i, iters, threads, v))
+
+                    for t in tasks:
+                        t.result()
+                    tasks.clear()
                     print(
                         colored(
                             f'set/{n}'.ljust(12), color='blue', attrs='bold') +
@@ -618,8 +674,13 @@ def cli():
                     for n, v in [('numeric', 777.777), ('string', 'x' * 1000),
                                  ('array', test_arr), ('dict', test_dict)]:
                         start = time.perf_counter()
-                        for z in range(iters):
-                            db.get(key=f'.benchmark/{n}/key{z}')
+                        for z in range(threads):
+                            tasks.append(
+                                p.submit(_bm_get_key, db, n, i, iters, threads))
+
+                        for t in tasks:
+                            t.result()
+                        tasks.clear()
                         print(
                             colored(f'get{"(cached)" if c else ""}/{n}'.ljust(
                                 12),
@@ -647,8 +708,6 @@ def cli():
     need_launch = len(
         sys.argv) > 1 or not db_path or str(db_path).startswith('-')
 
-    ap = icli.ArgumentParser()
-
     sp = ap.add_subparsers(dest='cmd')
 
     ap_get = sp.add_parser('get', help='Get key value')
@@ -714,16 +773,29 @@ def cli():
                        help='Include hidden')
 
     ap_dump = sp.add_parser('dump', help='Dump key and its subkeys')
-    ap_dump.add_argument('KEY', help='Key name').completer = KeyGroupCompleter()
-    ap_dump.add_argument('FILE', help='File name ("-" for stdout)')
+    sp_dump = ap_dump.add_subparsers(dest='_func')
 
-    ap_dump = sp.add_parser('load', help='Load dumped keys')
-    ap_dump.add_argument('FILE', help='File name ("-" for stdin)')
+    ap_dump_save = sp_dump.add_parser('save', help='Save key dump')
+
+    ap_dump_save.add_argument('KEY',
+                              help='Key name').completer = KeyGroupCompleter()
+    ap_dump_save.add_argument('FILE', help='File name ("-" for stdout)')
+
+    ap_dump_load = sp_dump.add_parser('load', help='Load dumped keys')
+    ap_dump_load.add_argument('FILE', help='File name ("-" for stdin)')
+
+    ap_dump_view = sp_dump.add_parser('view', help='View dumped keys')
+    ap_dump_view.add_argument('FILE', help='File name ("-" for stdin)')
+    ap_dump_view.add_argument('-y', '--full', action='store_true')
 
     ap_info = sp.add_parser('info', help='Database info')
     ap_info.add_argument('-y', '--full', action='store_true')
 
-    ap_purge = sp.add_parser('benchmark', help='Benchmark database')
+    ap_benchmark = sp.add_parser('benchmark', help='Benchmark database')
+    ap_benchmark.add_argument('--threads',
+                              metavar='NUMBER',
+                              type=int,
+                              default=4)
 
     ap_check = sp.add_parser('check', help='Check database')
 
@@ -743,7 +815,7 @@ def cli():
         'NEW_FORMAT', choices=['json', 'cbor', 'msgpack', 'yaml', 'pickle'])
     ap_convert.add_argument('--disable-checksums', action='store_true')
 
-    ap.ps = '{}> '.format(colored(db_ps, color='yellow'))
+    ap.ps = '{}> '.format(colored(os.getenv('YEDB_PS', db_ps), color='yellow'))
 
     ap.run = dispatcher
 
